@@ -103,18 +103,13 @@ eel_pango_layout_set_font_desc_from_string (PangoLayout *layout, const char *str
 
 /* Caution: this is an _expensive_ function */
 static int
-measure_string_width (const char    *string,
-		      PangoContext  *context,
-		      PangoAttrList *list)
+measure_string_width (const char  *string,
+		      PangoLayout *layout)
 {
-	PangoLayout *layout;
 	int width;
 	
-	layout = pango_layout_new (context);
 	pango_layout_set_text (layout, string, -1);
-	pango_layout_set_attributes (layout, list);
 	pango_layout_get_pixel_size (layout, &width, NULL);
-	g_object_unref (G_OBJECT (layout));
 
 	return width;
 }
@@ -122,8 +117,7 @@ measure_string_width (const char    *string,
 /* this is also plenty slow */
 static void
 compute_character_widths (const char    *string,
-			  PangoContext  *context,
-			  PangoAttrList *list,
+			  PangoLayout   *layout,
 			  int           *char_len_return,
 			  int          **widths_return,
 			  int          **cuts_return)
@@ -135,7 +129,6 @@ compute_character_widths (const char    *string,
 	int byte_len;
 	const char *p;
 	int i;
-	PangoLayout *layout;
 	PangoLayoutIter *iter;
 	PangoLogAttr *attrs;
 	
@@ -164,9 +157,7 @@ compute_character_widths (const char    *string,
 	}
 
 	/* Now fill in the widths array */
-	layout = pango_layout_new (context);
 	pango_layout_set_text (layout, string, -1);
-	pango_layout_set_attributes (layout, list);
 	
 	iter = pango_layout_get_iter (layout);
 
@@ -188,7 +179,6 @@ compute_character_widths (const char    *string,
 	} while (pango_layout_iter_next_char (iter));
 
 	pango_layout_iter_free (iter);
-	g_object_unref (G_OBJECT (layout));
 
 	g_free (offsets);
 	
@@ -200,7 +190,8 @@ compute_character_widths (const char    *string,
 	attrs = g_new (PangoLogAttr, char_len + 1);
 	
 	pango_get_log_attrs (string, byte_len, -1,
-			     pango_context_get_language (context),
+			     pango_context_get_language (
+				     pango_layout_get_context (layout)),
 			     attrs,
 			     char_len + 1);
 
@@ -229,12 +220,7 @@ eel_string_ellipsize_start (const char *string, PangoLayout *layout, int width)
 	int char_len;
 	const char *p;
 	int truncate_offset;
-	PangoAttrList *list;
-	PangoContext *context;
 
-	context = pango_layout_get_context (layout);
-	list = pango_layout_get_attributes (layout);
-	
 	/* Zero-length string can't get shorter - catch this here to
 	 * avoid expensive calculations
 	 */
@@ -245,7 +231,7 @@ eel_string_ellipsize_start (const char *string, PangoLayout *layout, int width)
 	 * to just dump this, and always do the compute_character_widths() etc.
 	 * down below.
 	 */
-	resulting_width = measure_string_width (string, context, list);
+	resulting_width = measure_string_width (string, layout);
 
 	if (resulting_width <= width) {
 		/* String is already short enough. */
@@ -253,7 +239,7 @@ eel_string_ellipsize_start (const char *string, PangoLayout *layout, int width)
 	}
 
 	/* Remove width of an ellipsis */
-	width -= measure_string_width (ELLIPSIS, context, list);
+	width -= measure_string_width (ELLIPSIS, layout);
 
 	if (width < 0) {
 		/* No room even for an ellipsis. */
@@ -271,7 +257,7 @@ eel_string_ellipsize_start (const char *string, PangoLayout *layout, int width)
 	 * in "string"
 	 */
 
-	compute_character_widths (string, context, list, &char_len, &widths, &cuts);
+	compute_character_widths (string, layout, &char_len, &widths, &cuts);
 
         for (truncate_offset = 1; truncate_offset < char_len; truncate_offset++) {
 
@@ -301,30 +287,25 @@ eel_string_ellipsize_end (const char *string, PangoLayout *layout, int width)
 	const char *p;
 	int truncate_offset;
 	char *result;
-	PangoAttrList *list;
-	PangoContext *context;
-
-	context = pango_layout_get_context (layout);
-	list = pango_layout_get_attributes (layout);
 	
 	/* See explanatory comments in ellipsize_start */
 	
 	if (*string == '\0')
 		return g_strdup ("");
 
-	resulting_width = measure_string_width (string, context, list);
+	resulting_width = measure_string_width (string, layout);
 	
 	if (resulting_width <= width) {
 		return g_strdup (string);
 	}
 
-	width -= measure_string_width (ELLIPSIS, context, list);
+	width -= measure_string_width (ELLIPSIS, layout);
 
 	if (width < 0) {
 		return g_strdup ("");
 	}
 	
-	compute_character_widths (string, context, list, &char_len, &widths, &cuts);
+	compute_character_widths (string, layout, &char_len, &widths, &cuts);
 	
         for (truncate_offset = char_len - 1; truncate_offset > 0; truncate_offset--) {
         	resulting_width -= widths[truncate_offset];
@@ -358,31 +339,25 @@ eel_string_ellipsize_middle (const char *string, PangoLayout *layout, int width)
 	int starting_fragment_length;
 	int ending_fragment_offset;
 	char *result;
-	PangoAttrList *list;
-	PangoContext *context;
-
-	context = pango_layout_get_context (layout);
-	list = pango_layout_get_attributes (layout);
-
 	
 	/* See explanatory comments in ellipsize_start */
 	
 	if (*string == '\0')
 		return g_strdup ("");
 
-	resulting_width = measure_string_width (string, context, list);
+	resulting_width = measure_string_width (string, layout);
 	
 	if (resulting_width <= width) {
 		return g_strdup (string);
 	}
 
-	width -= measure_string_width (ELLIPSIS, context, list);
+	width -= measure_string_width (ELLIPSIS, layout);
 
 	if (width < 0) {
 		return g_strdup ("");
 	}
 	
-	compute_character_widths (string, context, list, &char_len, &widths, &cuts);
+	compute_character_widths (string, layout, &char_len, &widths, &cuts);
 	
 	starting_fragment_length = char_len / 2;
 	ending_fragment_offset = starting_fragment_length + 1;
@@ -622,7 +597,7 @@ eel_self_check_ellipsize (const char *string, const char *truncate_to_length_str
 	layout = pango_layout_new (context);
 	
 	/* measure the length we want to truncate to */
-	truncation_length = measure_string_width (truncate_to_length_string, context, NULL);
+	truncation_length = measure_string_width (truncate_to_length_string, layout);
 
 	eel_pango_layout_set_text_ellipsized (layout, string, truncation_length, mode);
 
